import asyncio
import serial_asyncio
import time
import serial
import datetime
from os.path import isfile
import OPi.GPIO as GPIO
import os

# copy authorization to apache tohave priviledges as root
os.system("cp /var/www/html/cgi-bin/python_web /etc/sudoers.d/python_web")

GPIO.setboard(GPIO.ZERO)
GPIO.setmode(GPIO.BOARD)  #refer as pin number
GPIO.setwarnings(False)

Pin_LED = 23 # pin 23 or PA14

HOST = '127.0.0.1'  # The server's hostname or IP address
PORT = 8199        # The port used by the local server

uartGPS = "/dev/ttyACM0"
uartNANO = "/dev/ttyUSB0"
writeToNano = b'' 
bufNANO = bytes()
bufGPS = bytes()

GPS = serial.Serial(uartGPS, 9600, timeout=0.02)
NANO = serial.Serial(uartNANO, 9600, timeout=0.02)

idxNANO=0
idxGPS=0
dataNANO=[]
dataGPS=[]
idxRefresh=0

file_data_base = "/var/www/html/PY/gpsdo.db"
Osc1 = str(10000000) #Default nominal frequency
Osc2 = str(40000000)
vOsc1 = str(6)       #Default amplitude
vOsc2 = str(6)
XtalCor = str(0)      #Default Cristal correction
TimeLastPulseDiff = time.time()
TimeLastGPS_Unlock = 0

def Save_Nominal_Data() :
    global Osc1
    global Osc2
    global vOsc1
    global vOsc2
    global XtalCor
    file_handle=open(file_data_base,"w")
    file_handle.write(Osc1 + "\n")
    file_handle.write(Osc2 + "\n")
    file_handle.write(vOsc1 + "\n")
    file_handle.write(vOsc2 + "\n")
    file_handle.write(XtalCor + "\n")
    file_handle.close()
    
    
def Refresh_Freq_to_Nano() :
    global Osc1
    global Osc2
    global vOsc1
    global vOsc2
    global XtalCor
    global idxRefresh
    global writeToNano
    if   writeToNano ==b'':
        if idxRefresh ==10 :
            writeToNano =bytes("a"+Osc1,'ascii') 
        if idxRefresh ==30 :
            writeToNano =bytes("b"+Osc2,'ascii') 
        if idxRefresh ==50 :
            writeToNano =bytes("c"+vOsc1,'ascii') 
        if idxRefresh ==70 :
            writeToNano =bytes("d"+vOsc2,'ascii')
       
        idxRefresh = (1+idxRefresh)%1000
        
        
def Read_data_Base() :
    global Osc1
    global Osc2
    global vOsc1
    global vOsc2
    global XtalCor
    global file_data_base
    file_handle=open(file_data_base,"r")
    Osc1 = file_handle.readline().strip()
    Osc2 = file_handle.readline().strip()
    vOsc1 = file_handle.readline().strip()
    vOsc2 = file_handle.readline().strip()
    XtalCor = file_handle.readline().strip()
    file_handle.close()
    
            
def MessageToNano() :
    global writeToNano
    if  writeToNano !=b'':
        print( "writeToNano",writeToNano ) 
        NANO.write(writeToNano + b'\n') 
        if writeToNano == b'R' :
            time.sleep(0.5)
            Read_data_Base()
            writeToNano = bytes("x"+XtalCor,'ascii')
            print( "writeToNano after reset",writeToNano )
            NANO.write(writeToNano + b'\n')
            
        writeToNano=b''

if not isfile(file_data_base):
    Save_Nominal_Data()
    


Read_data_Base()

print ("Osc1",Osc1,"Osc2",Osc2,"XtalCor",XtalCor)

for i in   range(100):
    dataNANO.append("")
    dataGPS.append("")
    
    
GPIO.setup(Pin_LED, GPIO.OUT)
Pin_LED_Status = 0

time.sleep(2.5)    
for i in   range(80):
    time.sleep(0.01)
    Refresh_Freq_to_Nano() #SET Frequencies
    MessageToNano()

time.sleep(0.03)  
writeToNano =bytes("x"+XtalCor,'ascii') # Set previous known correction
MessageToNano()


# Management of messages received internally from web client via status_gpsdo.py
async def handle_Local_Server(reader, writer): 
    global writeToNano
    global idxNANO
    global idxGPS
    global Osc1
    global Osc2
    global vOsc1
    global vOsc2
    global XtalCor
    data = await reader.read(500)
    message = data.decode().strip()
    addr = writer.get_extra_info('peername')

    #print(f"Received {message!r} from {addr!r}")
    
    if message[0:4]=="Nano":
        last =(1+ int( message[4:])) % 100
        data=b'';
        while last != idxNANO % 100:
            if dataNANO[last] !="" :
                data=data+bytes(dataNANO[last]+"&",'ascii')
            last = (1+last) % 100
        writer.write(data)
        await writer.drain()
        
    if message[0:7]=="GPS_Raw":
        last =(1+ int( message[7:])) % 100
        data=b'';
        while last != idxGPS % 100:
            if dataGPS[last] !="" :
                data=data+bytes(dataGPS[last]+"&",'ascii')
            last = (1+last) % 100
        writer.write(data)
        await writer.drain()
        
    if message[0:4]=="Osc1":
        val = message[4:]
        if int(val) >2499 :
            Osc1 = val        
            writeToNano =bytes("a"+val,'ascii') 
            writer.write(bytes("OK_Osc1:"+val,'ascii'))
            await writer.drain()
            Save_Nominal_Data()
        
    if message[0:4]=="Osc2":
        val = message[4:]
        if int(val) >2499 :
            Osc2 = val
            writeToNano =bytes("b"+val,'ascii') 
            writer.write(bytes("OK_Osc2:"+val,'ascii'))
            await writer.drain()
            Save_Nominal_Data()
        
    if message[0:5]=="vOsc1":
        val = message[5:]
        if int(val) >0 :
            vOsc1 = val
            writeToNano =bytes("c"+val,'ascii') 
            writer.write(bytes("OK_vOsc1:"+val,'ascii'))
            await writer.drain()
            Save_Nominal_Data()
        
    if message[0:5]=="vOsc2":
        val = message[5:]
        if int(val) >0 :
            vOsc2 = val
            writeToNano =bytes("d"+val,'ascii') 
            writer.write(bytes("OK_vOsc2:"+val,'ascii'))
            await writer.drain()
            Save_Nominal_Data()
        
    if message[0:7]=="XtalCor":
        val = message[7:]
        if val !="" :
            XtalCor = val
            writeToNano =bytes("x"+val,'ascii') 
            writer.write(bytes("OK_XtalCor:"+val,'ascii'))
            await writer.drain()
            Save_Nominal_Data()
        
    if message[0:5]=="Reset":
        writeToNano =bytes("r",'ascii') 
        writer.write(bytes("Reset Nano Counter",'ascii'))
        await writer.drain()
        
    if message[0:5]=="ReadG":
        writeToNano =bytes("G",'ascii') 
        writer.write(bytes("Read Nano Counter",'ascii'))
        await writer.drain()
        
    if message[0:7]=="RebootN":
        writeToNano =bytes("R",'ascii') 
        writer.write(bytes("Reboot Nano",'ascii'))
        await writer.drain()

    writer.close()

async def read_Serie():
    global bufGPS
    global bufNANO
    global idxNANO
    global idxGPS
    global dataNANO
    global dataGPS
    global XtalCor
    global writeToNano
    global Pin_LED_Status
    global TimeLastPulseDiff
    global TimeLastGPS_Unlock
    
    while True:
        await asyncio.sleep(0.001)
        
        # Read DATA from GPS
        # ******************
        data = GPS.readline()
        if data:
            bufGPS += data
            if b'\n' in bufGPS:
                lines = bufGPS.split(b'\n')
                bufGPS = lines[-1]  # whatever was left over
                for line in lines[:-1]:
                    L=line.decode().strip()
                    i = idxGPS % 100
                    d = datetime.datetime.now()
                    dataGPS[i]=str(idxGPS)+"|" + str(d) + "|"+L
                    idxGPS +=1 
                    f = L[0:6].find("RMC")      
                    if f>0 :
                        tab= L.split(",")
                        Pin_LED_Status = (Pin_LED_Status +1) % 2
                        if tab[2] == "V"  :    # GPS not locked
                            Pin_LED_Status = 1                          # LED ON if GPS not locked
                            TimeLastGPS_Unlock = time.time()
                            if  writeToNano ==b'' :    
                                writeToNano = bytes("r",'ascii')        # reset Nano counter
                                
                        GPIO.output(Pin_LED, Pin_LED_Status)  
                    Refresh_Freq_to_Nano() #Periodic refresh
        
        # Read DATA from arduino NANO
        # ***************************
        data = NANO.readline()
        if data:
            bufNANO += data
            if b'\n' in bufNANO:
                lines = bufNANO.split(b'\n')
                bufNANO = lines[-1]  # whatever was left over
                for line in lines[:-1]:
                    L=line.decode().strip()
                    #print(f'NANO received: {L}')
                    i = idxNANO % 100
                    d = datetime.datetime.now()
                    dataNANO[i]=str(idxNANO)+"|" + str(d) + "|"+L
                    idxNANO +=1
                    f = L.find("PulseDiff:")
                    if f>=0 :
                        f = L.find(":")
                        g = L.find(" ")
                        val=L[f+1:g]
                        if int(val) > - 10000000 : # Pulse couter seems to work
                            TimeLastPulseDiff=time.time() #Receive a correct Pulse Difference
                    f = L.find("XtalCorrection:")
                    if f>=0 :
                        f = L.find(":")
                        g = L.find(" ")
                        val=L[f+1:g]
                        if int(val) != 0 : # good estimate correction value
                            XtalCor = val.strip()
                            Save_Nominal_Data()
                            
        # Test if Nano not blocked or no GPS 1s clock
        if time.time() > ( TimeLastPulseDiff+45) and time.time() > (TimeLastGPS_Unlock +45) : # GPS OK but no coorect Pulse Diff received
            TimeLastPulseDiff=time.time()
            print("Force Arduino Reboot")
            writeToNano =bytes("R",'ascii') # Force Arduino Reboot
            
            
        
        # Send any message to arduino NANO
        # ***************************        
        MessageToNano()
        
       
        
              
        
            

            
async def main_Local_Server():
    
    server = await asyncio.start_server(
       handle_Local_Server, HOST, PORT)

    addr = server.sockets[0].getsockname()
    print(f'Serving on {addr}')

    await read_Serie()
    
    async with server:        
        await server.serve_forever()
        



loop = asyncio.get_event_loop()
loop.run_until_complete(main_Local_Server())

loop.run_forever()
loop.close()